import React, { Component } from 'react';
import PropTypes from 'prop-types';
import 'date-fns';
import DateFnsUtils from '@date-io/date-fns';
import { createMuiTheme } from '@material-ui/core';
import { ThemeProvider } from '@material-ui/styles';
import { MuiPickersUtilsProvider, TimePicker, DatePicker, DateTimePicker as MuiDateTimePicker } from '@material-ui/pickers';

// Styles
import getStyles from './DateTimePicker.styles';

/**
 * DateTimePicker component for picking date and time from user
 * @see https://material-ui.com/demos/pickers/#material-ui-pickers
 * @see https://material-ui-pickers.dev/getting-started/usage
 */

class DateTimePicker extends Component {
    constructor(props) {
        super(props);
        this.state = {
            selectedDate: props.selectedDate || null,
        };
    }

    render() {
        const {
            contextStyles,
            format,
            is24Hour,
            mode,
            testID,
        } = this.props;

        const { selectedDate } = this.state;

        const styles = getStyles(contextStyles);

        const theme = createMuiTheme({
            overrides: {
                MuiFormControl: {
                    root: {
                        ...styles.container,
                    },
                },
                MuiInputBase: {
                    root: {
                        ...styles.inputContainer,
                    },
                    input: {
                        ...styles.inputText,
                    },
                },
                MuiIconButton: {
                    root: {
                        ...styles.iconContainer,
                    },
                    label: {
                        ...styles.icon,
                    },
                },
                MuiPickersToolbar: {
                    toolbar: {
                        ...styles.pickerToolbar,
                    },
                },
                MuiPickerDTTabs: {
                    tabs: {
                        ...styles.pickerTabs,
                    },
                },
                MuiPickersModal: {
                    // TODO: what does this refer to?  Changing seems to have no effects.
                    dialog: {
                        ...styles.pickerModalDialog,
                    },

                    // TODO: changed from 'dialogAction' due to reported error.  This does not seem to affect 'actions'.
                    dialogRoot: {
                        ...styles.pickerModalDialogAction,
                    },
                },
                MuiPickersCalendarHeader: {
                    switchHeader: {
                        ...styles.calenderHeader,
                    },
                },
                MuiPickersDay: {
                    day: {
                        ...styles.calenderDay,
                    },
                    isSelected: {
                        ...styles.calenderDayIsSelected,
                    },
                },
                MuiPickersClock: {
                    container: {
                        ...styles.clockContainer,
                    },
                    clock: {
                        ...styles.clock,
                    },
                    pin: {
                        ...styles.clockPin,
                    },
                },
                MuiPickersClockNumber: {
                    clockNumber: {
                        ...styles.clockNumber,
                    },
                    '&$selected': {
                        ...styles.clockNumberSelected,
                    },
                },
                MuiPickersClockPointer: {
                    pointer: {
                        ...styles.clockPointer,
                    },
                    thumb: {
                        ...styles.clockThumb,
                    },
                    noPoint: {
                        ...styles.clockNoPoint,
                    },
                },
            },
            typography: {
                useNextVariants: true,
            },
        });

        const dateProps = {
            keyboard: true,
            'data-testid': testID,
            value: selectedDate,
            ampm: !is24Hour,
            okLabel: 'Set',
            format,
            onChange: this.handleChange,
        };

        return (
            <MuiPickersUtilsProvider utils={ DateFnsUtils }>
                <ThemeProvider theme={ theme }>
                    { mode === 'date' &&
                    <DatePicker { ...dateProps } />
                    }
                    { mode === 'time' &&
                    <TimePicker { ...dateProps } />
                    }
                    { mode === 'datetime' &&
                    <MuiDateTimePicker { ...dateProps } />
                    }
                </ThemeProvider>
            </MuiPickersUtilsProvider>
        );
    }

    componentDidUpdate(prevProps, prevState) {
        const { selectedDate } = this.props;

        // Update the selectedDate state when 'selectedDate' prop is different
        if (selectedDate !== prevState.selectedDate) {
            this.updateDate(selectedDate);
        }
    }

    /**
    * Helper method to update date
    * @param {{target: Object}} selectedDate - The new value
    */
    updateDate = (selectedDate) => {
        this.setState({ selectedDate });
    };

    /**
    * Called when the input changes
    * @param {{target: Object}} selectedDate - The new value
    */
    handleChange = (selectedDate) => {
        const { onChange } = this.props;
        this.updateDate(selectedDate);
        if (onChange) {
            onChange(selectedDate);
        }
    };
}

DateTimePicker.propTypes = {
    /** Styles for this component */
    contextStyles: PropTypes.shape({
        /** Styles for the container surrounding the label and picker */
        container: PropTypes.object,

        /** Styles for the container surrounding the input */
        inputContainer: PropTypes.object,

        /** Styles for the input text */
        inputText: PropTypes.object,

        /** Styles for the container surrounding the icon */
        iconContainer: PropTypes.object,

        /** Styles for the icon */
        icon: PropTypes.object,

        /** Styles for the picker toolbar */
        pickerToolbar: PropTypes.object,

        /** Styles for the picker tabs */
        pickerTabs: PropTypes.object,

        /** Styles for the picker modal dialog */
        pickerModalDialog: PropTypes.object,

        /** Styles for the picker modal action buttons*/
        pickerModalDialogAction: PropTypes.object,

        /** Styles for the calender header */
        calenderHeader: PropTypes.object,

        /** Styles for the calender day */
        calenderDay: PropTypes.object,

        /** Styles for the selected calender day */
        calenderDayIsSelected: PropTypes.object,

        /** Styles for the clock container */
        clockContainer: PropTypes.object,

        /** Styles for the clock */
        clock: PropTypes.object,

        /** Styles for the clock pin */
        clockPin: PropTypes.object,

        /** Styles for the clock pointer */
        clockPointer: PropTypes.object,

        /** Styles for the clock thumb */
        clockThumb: PropTypes.object,

        /** Styles for the clock pointer end*/
        clockNoPoint: PropTypes.object,

        /** Styles for the clock number */
        clockNumber: PropTypes.object,

        /** Styles for the clock number selected */
        clockNumberSelected: PropTypes.object,
    }),

    /** Custom display format (see NPM package date-fns) */
    format: PropTypes.string,

    /** Display the date picker in a 24 hour format */
    is24Hour: PropTypes.bool,

    /** Date type format: date, time, datetime */
    mode: PropTypes.oneOf([
        'date',
        'time',
        'datetime',
    ]),

    /** Selected date value */
    selectedDate: PropTypes.instanceOf(Date),

    /**
    * Called when the picker date has changed
    * @param {Date} date - The selected date
    */
    onChange: PropTypes.func,

    /** Id used for testing */
    testID: PropTypes.string,
};

DateTimePicker.defaultProps = {
    contextStyles: {},
    format: 'M/dd/yy, h:mm a',
    mode: 'date',
};

export default DateTimePicker;
